//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-word-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using System.Linq;
using System.Globalization;
using GrapeCity.Documents.Word;

namespace DsWordWeb.Demos
{
    // This example shows how to deal with the 'data source with this name already exists' error.
    public class DataTplFixSpecifyDataSource
    {
        // Code demonstrating the problem:
        GcWordDocument Problem()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            using var oceans1 = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.DataTemplate.DataSources.Add("ds", oceans);
            doc.DataTemplate.DataSources.Add("ds1", oceans1);
            // Incorrect: template tag is used without specifying the data source part.
            // It is allowed only if the template engine is able to resolve the tags.
            // But because in this case there are two data sources that both contain a field called 'name',
            // if causes an exception as the tag cannot be resolved:
            doc.Body.Paragraphs.Add("{{name}}");
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        // Code demonstrating the fix:
        GcWordDocument Fix()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            using var oceans1 = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.DataTemplate.DataSources.Add("ds", oceans);
            doc.DataTemplate.DataSources.Add("ds1", oceans1);
            // Correct: use fully qualified data tags so there is no ambiguity:
            doc.Body.Paragraphs.Add("{{ds.name}}");
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        public GcWordDocument CreateDocx()
        {
            GcWordDocument doc;
            try
            {
                // This fails:
                doc = Problem();
            }
            catch (Exception ex)
            {
                // This works:
                doc = Fix();
                // Insert a brief explanation of the problem and the fix into the generated document:
                doc.Body.Paragraphs.Insert(
                    $"The error \"{ex.Message}\" occurred because the data template engine could not unambiguously resolve the data tag " +
                    $"that was not fully qualified. " +
                    $"The fix is to use fully qualified data tags.",
                    doc.Styles[BuiltInStyleId.BlockText],
                    InsertLocation.Start);
            }
            return doc;
        }
    }
}
